﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using Sample.Class;
using ThinkGeo.MapSuite.DesktopEdition;
using PdfSharp.Pdf;
using PdfSharp;
using System.Collections.ObjectModel;
using System.Diagnostics;

namespace Sample
{
    public partial class Form2 : Form
    {
        ManagedProj4Projection _proj = null;

        public Form2()
        {
            InitializeComponent();
        }

        private void Form2_Load(object sender, EventArgs e)
        {
            try
            {
                _proj = new ManagedProj4Projection();
                _proj.InternalProjectionParametersString = ManagedProj4Projection.GetEpsgParametersString(4326);
                _proj.ExternalProjectionParametersString = ManagedProj4Projection.GetEsriParametersString(54004);
                LoadDefaultMap();

                PlotRoute();

                //wfMap.CurrentExtent = _proj.ConvertToExternalProjection(new RectangleShape(-130, 60, 130, -60));
                if (wfMap.FindFeatureLayer("RouteLayer") != null)
                {
                    wfMap.FindFeatureLayer("RouteLayer").Open();
                    wfMap.CurrentExtent = wfMap.FindFeatureLayer("RouteLayer").GetBoundingBox();
                    wfMap.FindFeatureLayer("RouteLayer").Close();
                }
                wfMap.Refresh();
            }
            catch (Exception)
            {
                throw;
            }
        }

        public void LoadDefaultMap()
        {
            try
            {
                _proj.Open();
                wfMap.MapUnit = _proj.GetExternalGeographyUnit();
                wfMap.ExtentOverlay.MouseWheelMode = MapMouseWheelMode.Disabled;
                wfMap.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.FromArgb(255, 200, 227, 255));

                ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(Application.StartupPath + @"/Data/Countries02.shp");
                if (_proj != null)
                    worldLayer.FeatureSource.Projection = _proj;
                worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = new AreaStyle(new GeoPen(GeoColor.FromArgb(255, 128, 128, 64), 1F), new GeoSolidBrush(GeoColor.FromArgb(255, 239, 228, 176)));
                worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                LayerOverlay worldOverlay = new LayerOverlay();
                worldOverlay.Layers.Add("WorldLayer", worldLayer);
                wfMap.Overlays.Add("WorldOverlay", worldOverlay);
            }
            catch (Exception)
            {
            }
        }

        private void PlotRoute()
        {
            try
            {
                InMemoryFeatureLayer imRouteLayer = new InMemoryFeatureLayer();
                imRouteLayer.Open();
                imRouteLayer.FeatureSource.Projection = _proj;
                imRouteLayer.Columns.Add(new FeatureSourceColumn("Text"));

                imRouteLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = new PointStyle(PointSymbolType.Circle, new GeoSolidBrush(GeoColors.Transparent), new GeoPen(GeoColors.Black), 5);
                imRouteLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = new LineStyle(new GeoPen(GeoColors.Black));
                imRouteLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = new TextStyle("Text", new GeoFont("Arial", 10, DrawingFontStyles.Bold), new GeoSolidBrush(GeoColors.Black));
                imRouteLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.BestPlacement = true;
                imRouteLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                Feature ft=new Feature(new PointShape(10,10));
                ft.ColumnValues.Add("Text","RIYAS");
                imRouteLayer.InternalFeatures.Add(ft);

                ft = new Feature(new PointShape(20, 20));
                ft.ColumnValues.Add("Text", "ASHOK");
                imRouteLayer.InternalFeatures.Add(ft);

                ft = new Feature(new LineShape("LINESTRING(10 10,20 20)"));
                ft.ColumnValues.Add("Text", "");
                imRouteLayer.InternalFeatures.Add(ft);


                ft = new Feature(new PointShape(10, 10));
                ft.ColumnValues.Add("Text", "ARYAN");
                imRouteLayer.InternalFeatures.Add(ft);

                ft = new Feature(new PointShape(30, -20));
                ft.ColumnValues.Add("Text", "VIVEK");
                imRouteLayer.InternalFeatures.Add(ft);

                ft = new Feature(new LineShape("LINESTRING(10 10,30 -20)"));
                ft.ColumnValues.Add("Text", "");
                imRouteLayer.InternalFeatures.Add(ft);

                LayerOverlay lo = new LayerOverlay();
                lo.Layers.Add("RouteLayer", imRouteLayer);
                wfMap.Overlays.Add("RouteOverlay", lo);
            }
            catch (Exception)
            {
                throw;
            }
        }

        private void btnPrint_Click(object sender, EventArgs e)
        {
            try
            {
                ExportToPdf(wfMap, true);
                //PdfGeoCanvas pdfGeoCanvas = new PdfGeoCanvas();
                ////pdfGeoCanvas.DrawingArea = new Rectangle(200, 50, 400, 400);
                //Collection<SimpleCandidate> labelsInLayers = new Collection<SimpleCandidate>();
                //foreach (Layer layer in ((LayerOverlay)wfMap.Overlays[0]).Layers)
                //{
                //    pdfGeoCanvas.BeginDrawing(page, wfMap.CurrentExtent, GeographyUnit.DecimalDegree);
                //    layer.Open();
                //    layer.Draw(pdfGeoCanvas, labelsInLayers);
                //    layer.Close();
                //    pdfGeoCanvas.EndDrawing();
                //}


                //PrinterInteractiveOverLay printerOverlay = new PrinterInteractiveOverLay();
                //wfMap.InteractiveOverlays.Add("PrintPreviewOverlay", printerOverlay);
                //wfMap.InteractiveOverlays.MoveToBottom("PrintPreviewOverlay");

                //PagePrinterLayer pagePrinterLayer = new PagePrinterLayer(PrinterPageSize.AnsiA, PrinterOrientation.Portrait);
                //pagePrinterLayer.Open();
                //printerOverlay.PrinterLayers.Add("PageLayer", pagePrinterLayer);
            }
            catch (Exception)
            {
                throw new Exception();
            }
        }

        public void ExportToPdf(WinformsMap wfMap, bool islandscape)
        {
            try
            {
                SaveFileDialog mySaveWindow = new SaveFileDialog();
                mySaveWindow.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Desktop);
                mySaveWindow.FileName = "MyMap";
                //mySaveWindow.Filter = "BMP|*.bmp|PNG|*.png|GIF|*.gif|TIFF|*.tif|JPEG|*.jpg";
                mySaveWindow.Filter = "Pdf Files | *.pdf";
                DialogResult result = mySaveWindow.ShowDialog();
                if (result == DialogResult.OK)
                {
                    PdfDocument document = new PdfDocument();
                    PdfDocument documentcopy = new PdfDocument();
                    PdfPage page = document.AddPage();
                    PdfPage pagecopy = documentcopy.AddPage();

                    if (islandscape == true)
                    {
                        page.Orientation = PageOrientation.Landscape;
                        pagecopy.Orientation = PageOrientation.Landscape;
                    }
                    else
                    {
                        page.Orientation = PageOrientation.Portrait;
                        pagecopy.Orientation = PageOrientation.Portrait;
                    }

                    page.Size = PageSize.A4;
                    pagecopy.Size = PageSize.A4;
                    pagecopy.Rotate = 90;

                    PdfGeoCanvas pdfGeoCanvas = new PdfGeoCanvas();
                    PdfGeoCanvas pdfGeoCanvascopy = new PdfGeoCanvas();
                    Collection<SimpleCandidate> labelsInLayers = new Collection<SimpleCandidate>();
                    Collection<SimpleCandidate> labelsInLayerscopy = new Collection<SimpleCandidate>();

                    foreach (LayerOverlay layeroverlay in wfMap.Overlays)
                    {
                        foreach (Layer layer in layeroverlay.Layers)
                        {
                            if (layeroverlay.IsVisible == true)
                            {
                                pdfGeoCanvas.BeginDrawing(page, ExtentHelper.GetDrawingExtent(wfMap.CurrentExtent, (float)(page.Width.Point), (float)(page.Height.Point)), wfMap.MapUnit);
                                pdfGeoCanvascopy.BeginDrawing(pagecopy, ExtentHelper.GetDrawingExtent(wfMap.CurrentExtent, (float)(pagecopy.Width.Point), (float)(pagecopy.Height.Point)), wfMap.MapUnit);
                                if (!layer.IsOpen)
                                    layer.Open();

                                layer.Draw(pdfGeoCanvas, labelsInLayers);
                                layer.Draw(pdfGeoCanvascopy, labelsInLayerscopy);

                                pdfGeoCanvas.EndDrawing();
                                pdfGeoCanvascopy.EndDrawing();
                            }
                        }

                        pdfGeoCanvas.Flush();
                        pdfGeoCanvascopy.Flush();
                    }

                    document.Save(mySaveWindow.FileName);
                    Process.Start(mySaveWindow.FileName);
                }
            }
            catch (Exception)
            {
                throw new Exception();
            }
        }

    }
}
